function htstego_errdiffbin(NSHARES, imparam, txtparam)
if nargin == 0
    NSHARES  = 4;
    imparam  = "airplane80";
    txtparam = 2048;
end

imfile = sprintf("%s_256gray", imparam);
impath = sprintf("cover_imgs/%s.png", imfile);
I      = double(imread(impath)) / 255;
[M, N] = size(I);

% convert ascii message to binary string
txtfile         = sprintf('payloads/payload%d.txt', txtparam);
messageAscii    = fileread(txtfile);
messageBinary   = dec2bin(messageAscii, 8);
messagePreFinal = messageBinary';
messageFinal    = messagePreFinal(:)';
mLen            = length(messageFinal);
messagePos      = 1;

% calculate embedding distance
embedDis = M*N / mLen;
if embedDis < 1
    fprintf('[%2d %9s %4d] message too long!\n', NSHARES, imparam, txtparam)
    return
end
embedDis = floor(embedDis);

normalOutput      = floyd(I); % floyd-steinberg error diffusion
stegoOutput       = zeros(M, N, NSHARES);
linearImage       = reshape(normalOutput', [1, M*N]);
linearStegoImages = zeros(NSHARES, M*N);
for i = 1:NSHARES
    linearStegoImages(i, :) = linearImage;
end

snrs = zeros(NSHARES, 2);
ssims = zeros(NSHARES, 1);

% iterate over pixels in image and embed bits of payload in them
for i=1:embedDis:M*N
    if messagePos <= mLen
        currentBit = messageFinal(messagePos);
        if currentBit == '0'
            stegoPixel = 0;
        elseif currentBit=='1'
            stegoPixel = 1;
        end
        if i+embedDis-1>M*N
            break
        end
        currentSet = linearImage(i:i+embedDis-1);
        stegoSet   = currentSet;
        [~, embedHere, ~]  = findEmbedPositionErrDiff(currentSet, stegoPixel);
        if embedHere == 0
            continue
        end
        stegoSet(embedHere)=stegoPixel;
        randomShare = 1+floor(rand()*NSHARES);
        linearStegoImages(randomShare, i:i+embedDis-1) = stegoSet;
        messagePos=messagePos+1;
    else
        break
    end
end

% write output images on disk and calculate snr, psnr, ssim
normalOutputPath = sprintf('output/%s_hterrdiffbin_regular.png', imfile);
imwrite(normalOutput, normalOutputPath, 'png');
stegoOutputPath = strings(1, NSHARES);
for i = 1:NSHARES
    stegoOutput(:, :, i) = reshape(linearStegoImages(i,:), [N M])';
    [snrs(i, 1), snrs(i, 2)] = psnr(stegoOutput(:, :, i), normalOutput);
    ssims(i) = ssim(normalOutput, stegoOutput(:,:,i));
    stegoOutputPath(i) = sprintf('output/%s_hterrdiffbin_stego_msg%d_%dof%d.png', imfile, txtparam, i, NSHARES);
    imwrite(stegoOutput(:, :, i), stegoOutputPath(i), 'png');
end

% display results on screen
fprintf('[erdbin] [%2d %10s %4d] [avg_snr: %7.4f] [avg_psnr: %7.4f] [avg_ssim: %6.4f]\n', NSHARES, imparam, txtparam, mean(snrs(:,2)), mean(snrs(:,1)), mean(ssims))
