function htstego_errdiffcol(NSHARES, imparam, txtparam)
if nargin == 0
    NSHARES  = 4;
    imparam  = "airplane80";
    txtparam = 2048;
end

imfile = sprintf("%s_256rgb", imparam);
impath = sprintf("cover_imgs/%s.png", imfile);
I = double(imread(impath)) / 255;
[M, N, ~] = size(I);

% convert ascii message to binary string
txtfile         = sprintf('payloads/payload%d.txt', txtparam);
messageAscii    = fileread(txtfile);
messageBinary   = dec2bin(messageAscii, 8);
messagePreFinal = messageBinary';
messageFinal    = messagePreFinal(:)';
mLen            = length(messageFinal);
messagePos      = 1;

% calculate embedding distance
embedDis = M*N / mLen;
if embedDis < 1
    fprintf('[%2d %9s %4d] message too long!\n', NSHARES, imparam, txtparam)
    return
end
embedDis = floor(embedDis);

% generate regular halftone image
% floyd-steinberg error diffusion
normalOutput        = zeros(M, N, 3);
normalOutput(:,:,1) = floyd(I(:,:,1));
normalOutput(:,:,2) = floyd(I(:,:,2));
normalOutput(:,:,3) = floyd(I(:,:,3));

% initialize stego halftone image
stegoOutput         = zeros(M, N, 3, NSHARES);
linearImage         = zeros(M * N, 3);
linearImage(:,1)    = reshape(normalOutput(:,:,1)', [1, M*N]);
linearImage(:,2)    = reshape(normalOutput(:,:,2)', [1, M*N]);
linearImage(:,3)    = reshape(normalOutput(:,:,3)', [1, M*N]);
linearStegoImages   = zeros(NSHARES, M*N, 3);
for i = 1:NSHARES
    for j = 1:3
        linearStegoImages(i, :, j) = linearImage(:, j);
    end
end

snrs = zeros(NSHARES, 2);
ssims = zeros(NSHARES, 1);

% iterate over pixels in image and embed bits of payload in them
for i=1:embedDis:M*N
    if messagePos <= mLen
        currentBit = messageFinal(messagePos);
        if currentBit == '0'
            stegoPixel = 0;
        elseif currentBit=='1'
            stegoPixel = 1;
        end
        
        randomChannel = ceil(rand()*3);
        
        if i+embedDis-1>M*N
            break
        end
        currentSet = linearImage(i:i+embedDis-1, randomChannel);
        stegoSet   = currentSet;
        [~, embedHere, ~]  = findEmbedPositionErrDiff(currentSet, stegoPixel);
        if embedHere == 0
            continue
        end
        stegoSet(embedHere)=stegoPixel;
        randomShare = 1+floor(rand()*NSHARES);
        linearStegoImages(randomShare, i:i+embedDis-1, randomChannel) = stegoSet;
        messagePos=messagePos+1;
    else
        break
    end
end

% write output images on disk and calculate snr, psnr, ssim
normalOutputPath = sprintf('output/%s_hterrdiffcol_regular.png', imfile);
imwrite(normalOutput, normalOutputPath, 'png');
stegoOutputPath = strings(1, NSHARES);
for i = 1:NSHARES
    I=zeros(M,N,3);
    for j = 1:3
        stegoOutput(:, :, j, i) = reshape(linearStegoImages(i,:,j), [N M])';
        I(:,:,j)=stegoOutput(:,:,j,i);
    end
    [snrs(i, 1), snrs(i, 2)] = psnr(stegoOutput(:, :, :, i), normalOutput);
    ssims(i) = ssim(normalOutput, I);
    stegoOutputPath(i) = sprintf('output/%s_hterrdiffcol_stego_msg%d_%dof%d.png', imfile, txtparam, i, NSHARES);
    imwrite(I, stegoOutputPath(i), 'png');
end

% display results on screen
fprintf('[erdcol] [%2d %10s %4d] [avg_snr: %7.4f] [avg_psnr: %7.4f] [avg_ssim: %6.4f]\n', NSHARES, imparam, txtparam, mean(snrs(:,2)), mean(snrs(:,1)), mean(ssims))
