function htstego_patcol(NSHARES, imparam, txtparam)
if nargin == 0
   NSHARES  = 4;
   imparam  = "airplane80";
   txtparam = 2048;
end

imfile    = sprintf('%s_256rgb',imparam);
impath    = sprintf('cover_imgs/%s.png', imfile);
I         = double(imread(impath));
[M, N, ~] = size(I);

InormalOutput      = zeros(M*3, N*3, 3);
IstegoOutput       = zeros(M*3, N*3, 3, NSHARES);

% convert ascii message to binary string
txtfile         = sprintf('payloads/payload%d.txt', txtparam);
messageAscii    = fileread(txtfile);
messageAscii    = messageAscii(1:end-1);
messageBinary   = dec2bin(messageAscii, 8);
messagePreFinal = messageBinary';
messageFinal    = messagePreFinal(:)';
mLen            = length(messageFinal);
messagePos      = 1;

% calculate embedding distance
iBlocks      = M*N;
bwBlocks     = countBWBlocks(I);
usableBlocks = iBlocks - bwBlocks;
embedDis     = usableBlocks / mLen;
if embedDis < 1
    fprintf('  [%2d %9s %4d %4d %6.2f] message too long!\n', NSHARES, imparam, txtparam, usableBlocks, mLen*100/usableBlocks)
    return
end
embedDis = floor(embedDis);

snrs = zeros(NSHARES, 2);
ssims = zeros(NSHARES, 1);
I = floor(I/26); % convert image for 10 patterns
patterns=[[0 0 0; 0 0 0; 0 0 0] ... % 0
    [0 1 0; 0 0 0; 0 0 0] ...       % 1
    [0 1 0; 0 0 0; 0 0 1] ...       % 2
    [1 1 0; 0 0 0; 0 0 1] ...       % 3
    [1 1 0; 0 0 0; 1 0 1] ...       % 4
    [1 1 1; 0 0 0; 1 0 1] ...       % 5
    [1 1 1; 0 0 1; 1 0 1] ...       % 6
    [1 1 1; 0 0 1; 1 1 1] ...       % 7
    [1 1 1; 1 0 1; 1 1 1] ...       % 8
    [1 1 1; 1 1 1; 1 1 1]];         % 9

% create regular output and copies of it as unprocessed stego outputs
for i=1:M
    for j=1:N
        for k=1:3
            p = I(i, j, k);
            InormalOutput((i-1)*3+1:(i-1)*3+3, (j-1)*3+1:(j-1)*3+3, k) = patterns(1:3,p*3+1:p*3+3);
        end
    end
end
for k=1:NSHARES
    IstegoOutput(:, :, :, k) = InormalOutput;
end

% temporarily linearize M,N image to M*N array for easier hiding
iLinearStegoOutput  = zeros(3, M*N*3, 3, NSHARES);
for i=1:3
    iLinearStegoOutput(:,:,i,1) = convertHalftoneToArray(IstegoOutput(:,:,i,1), N, M);
end
for i=2:NSHARES
    iLinearStegoOutput(:,:,:,i) = iLinearStegoOutput(:,:,:,1);
end

% iterate over blocks in image and embed bits of payload in them
loopCount=1;
for i=1:3*embedDis:3*M*N
    if messagePos<=mLen
        currentBit = messageFinal(messagePos);
        randomChannel = ceil(rand()*3);
        currentSet = iLinearStegoOutput(1:3,i:i+(3*embedDis-1), randomChannel, 1);
        [~, embedHere, ~] = findEmbedPositionPat(currentSet);
        if embedHere==0
            continue
        end
        patternHere=1+sum(sum(currentSet(:,3*embedHere-2:3*embedHere)));
        if currentBit=='0'
            newPattern = patternHere - 1;
        elseif currentBit=='1'
            newPattern = patternHere + 1;
        end
        currentSet(1:3,3*embedHere-2:3*embedHere)=patterns(1:3,3*newPattern-2:3*newPattern);
        
        shareToEmbedInto=1+floor(NSHARES*rand());
        iLinearStegoOutput(1:3,i:i+(3*embedDis-1),randomChannel, shareToEmbedInto) = currentSet;        
        messagePos=messagePos+1;
        loopCount=loopCount+1;
    else
        break
    end
end

% de-linearize stego images, write output images on disk and calculate snr, psnr, ssim
normalOutputPath=sprintf('output/%s_htpatcol_regular.png',imfile);
imwrite(InormalOutput, normalOutputPath,'png');
stegoOutputPath=strings(1, NSHARES);
for i = 1:NSHARES
    for j=1:3
        IstegoOutput(:,:,j,i)=convertHalftoneToMatrix(iLinearStegoOutput(:,:,j,i),N,M);
    end
    [snrs(i,1), snrs(i,2)] = psnr(IstegoOutput(:,:,:,i), InormalOutput);
    ssims(i) = ssim(InormalOutput, IstegoOutput(:,:,:,i));
    stegoOutputPath(i) = sprintf('output/%s_htpatcol_stego_msg%d_%dof%d.png', imfile, txtparam,i,NSHARES);
    imwrite(IstegoOutput(:,:,:,i), stegoOutputPath(i), 'png');
end

% display results on screen
fprintf('[patcol] [%2d %10s %4d] [avg_snr: %7.4f] [avg_psnr: %7.4f] [avg_ssim: %6.4f]\n', NSHARES, imparam, txtparam, mean(snrs(:,2)), mean(snrs(:,1)), mean(ssims))
